from collections import deque
from simpy import Event


class EventQueue:
    """
    Class for events happening in a processor
    """

    def __init__(self, env):
        self._env = env
        self._evt_deque = deque([])
        self.item_ready_event = self._env.event()

    def append(self, item):
        self._evt_deque.append(item)
        if not self.item_ready_event.triggered:
            self.item_ready_event.succeed()

    def clear_events(self, event_type):
        self._evt_deque = deque(
            [e for e in self._evt_deque if e[0] != event_type])

    def popleft(self):
        return self._evt_deque.popleft()

    def is_empty(self):
        return not bool(self._evt_deque)

    def get_deque(self):
        return self._evt_deque
