
from simso.core import Scheduler
from simso.schedulers import scheduler

@scheduler("simso.schedulers.C_FP", required_task_fields = [
        {'name': 'priority', 'type' : 'int', 'default' : '0' }
    ]
)
class C_FP(Scheduler):
    """ Clustered Fixed-Priority """
    def init(self):
        self.ready_list = [] 
        self.mappings_dict = {}
        for k,v in self.configuration.mapping_dict.items():
            task = next(t for t in self.task_list if t._task_info == k)
            cpu_list = [p for p in self.processors if p.identifier in [pi.identifier for pi in v]]
            self.mappings_dict[task] = cpu_list
            task.cpu = self.mappings_dict[task][0]


    def on_activate(self, job):
        self.ready_list.append(job)
        job.cpu.resched()

    def on_terminated(self, job):
        if job in self.ready_list:
            self.ready_list.remove(job)
        else:
            job.cpu.resched()

    def schedule(self, cpu):
        if self.ready_list:
            # Get the job with the highest priority.
            job = max(self.ready_list, key=lambda x: x.data['priority'])

            # Get a free processor or a processor running a low priority job.
            key = lambda x: (
                1 if x.running else 0,
                x.running.data['priority'] if x.running else 0,
                0 if x is cpu else 1
            )
            cpu_min = min(self.mappings_dict[job.task], key=key)

            if (cpu_min.running is None or
                    cpu_min.running.data['priority'] < job.data['priority']):
                self.ready_list.remove(job)
                if cpu_min.running:
                    self.ready_list.append(cpu_min.running)
                return (job, cpu_min)

        return None